#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import logging
logger = logging.getLogger('PyAMS (catalog)')

# import interfaces
from hypatia.interfaces import ICatalog
from persistent.interfaces import IPersistent
from pyams_utils.interfaces.site import ISiteGenerations
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectRemovedEvent, IObjectModifiedEvent
from zope.site.interfaces import INewLocalSite

# import packages
from hypatia.catalog import Catalog
from pyams_catalog.utils import index_object, reindex_object, unindex_object
from pyams_utils.registry import utility_config, get_utilities_for
from pyams_utils.site import check_required_utilities
from pyramid.events import subscriber
from zope.interface.interface import InterfaceClass
from zope.intid import IntIds


REQUIRED_UTILITIES = ((IIntIds, '', IntIds, 'Internal IDs'),
                      (ICatalog, '', Catalog, 'Catalog'),)


@subscriber(INewLocalSite)
def handle_new_local_site(event):
    """Create a new catalog when a site is created"""
    site = event.manager.__parent__
    check_required_utilities(site, REQUIRED_UTILITIES)


@utility_config(name='PyAMS catalog', provides=ISiteGenerations)
class CatalogGenerationsChecker(object):
    """Catalog generations checker"""

    generation = 1

    def evolve(self, site, current=None):
        """Check for required utilities"""
        check_required_utilities(site, REQUIRED_UTILITIES)


def check_required_indexes(site, indexes, catalog_name=''):
    """Utility function to check for required catalog indexes

    utilities argument is a tuple made of:
    - the index name
    - the index class
    - index factory arguments
    """
    sm = site.getSiteManager()
    catalog = sm.queryUtility(ICatalog, name=catalog_name)
    if catalog is None:
        logger.warning("No catalog found! Index check ignored...")
        return
    for name, klass, args in indexes:
        index = catalog.get(name)
        if index is None:
            for key, value in args.copy().items():
                if callable(value) and not isinstance(value, InterfaceClass):
                    args[key] = value()
            index = klass(**args)
            catalog[name] = index


@subscriber(IObjectAddedEvent, context_selector=IPersistent)
def handle_new_object(event):
    """Index new persistent object"""
    for name, catalog in get_utilities_for(ICatalog):
        index_object(event.object, catalog, ignore_notyet=True)


@subscriber(IObjectModifiedEvent, context_selector=IPersistent)
def handle_modified_object(event):
    """Update catalog for modified object"""
    for name, catalog in get_utilities_for(ICatalog):
        reindex_object(event.object, catalog)


@subscriber(IObjectRemovedEvent, context_selector=IPersistent)
def handle_removed_object(event):
    """Un-index removed object"""
    for name, catalog in get_utilities_for(ICatalog):
        unindex_object(event.object, catalog)
